<?php

/**
 * @file
 * Provides an API for the lazy vars backend.
 */

/**
 * Provides an API for persistent variables that should be loaded on demand.
 */
class LazyVars implements LazyVarsInterface {

  /**
   * The storage table.
   */
  const TABLE = 'lazy_vars';

  /**
   * A static cache container.
   * @var array
   */
  private static $cached_vars = array();

  /**
   * Sets a persistent variable into the storage table.
   *
   * @param string $var_name
   *   The name of the variable to set.
   * @param mixed $value
   *   The value to set. This can be scalar or compound data type as the data
   *   will be serialized.
   * @param bool $cache
   *   Decides if the var-value pair should be added to the static cache or not.
   */
  public static function set($var_name, $value, $cache = FALSE) {
    db_query("UPDATE {" . self::TABLE . "} SET value = '%s' WHERE name = '%s'", serialize($value), $var_name);
    if (!db_affected_rows()) {
      @db_query("INSERT INTO {" . self::TABLE . "} (name, value) VALUES ('%s', '%s')", $var_name, serialize($value));
    }

    if ($cache) {
      self::$cached_vars[$var_name] = $value;
    }
  }

  /**
   * Returns a persistent variable.
   *
   * @param string $name
   *   The name of the variable to return.
   * @param mixed $default
   *   The default value to use if this variable has never been set.
   * @param bool $cache
   *   Decides if the var-value pair should be added to the static cache or not
   *   after it has been retrieved from the database.
   *
   * @return
   *   The value of the variable.
   */
  public static function get($var_name, $default = NULL, $cache = TRUE) {
    $value = '';
    if (isset(self::$cached_vars[$var_name])) {
      return self::$cached_vars[$var_name];
    }
    else {
      $query = "SELECT cs.value FROM {" . self::TABLE . "} cs
                  WHERE name = '%s'";

      $result = db_fetch_array(db_query($query, $var_name));

      if ($result) {
        $value = unserialize($result['value']);
        if ($cache) {
          self::$cached_vars[$var_name] = $value;
        }
        return $value;
      }
      else {
        return $default;
      }
    }
  }

  /**
   * Unsets a persistent variable.
   *
   * @param string $name
   *   The name of the variable to delete.
   */
  public static function del($var_name) {
    db_query("DELETE FROM " . self::TABLE . " WHERE name = '%s'", $var_name);

    if (isset(self::$cached_vars[$var_name])) {
      unset(self::$cached_vars[$var_name]);
    }
  }

}